<?php
// Prevent direct file access
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * This class protects categories (and their products) inside product loops, the main shop page,
 * search results, widgets, navigation menus, and in WooCommerce shortcodes.
 *
 * @package   WooCommerce_Protected_Categories
 * @author    Barn2 Media <info@barn2.co.uk>
 * @license   GPL-3.0
 * @copyright Barn2 Media Ltd
 */
class WC_PPC_Category_Protector {

	private static $hidden_categories	 = false;
	private static $unlocked_categories	 = false;

	// Hooks for filtering posts, categories, nav menus, etc
	public static function register_protection_hooks() {

		// Adjust query to exclude products in private or protected categories
		add_action( 'pre_get_posts', array( __CLASS__, 'protect_products_in_loop' ) );

		// Protect products in WC shortcodes
		add_filter( 'woocommerce_shortcode_products_query', array( __CLASS__, 'protect_products_in_shortcode' ), 10, 3 );

		// Protect product categories, e.g. in product categories widget
		add_filter( 'get_terms_args', array( __CLASS__, 'protect_product_categories' ), 10, 2 );

		// Adjust category counts - priority 15 to run after 'wc_change_term_counts' filter
		add_filter( 'get_terms', array( __CLASS__, 'update_category_counts' ), 15, 3 );

		// Protect categories and products in nav menus
		add_filter( 'wp_get_nav_menu_items', array( __CLASS__, 'protect_nav_menus' ), 10, 3 );

		// Ensure protected products are not purchasable
		add_filter( 'woocommerce_is_purchasable', array( __CLASS__, 'product_is_purchasable' ), 10, 2 );
	}

	public static function protect_products_in_loop( &$query ) {

		if ( isset( $query->query_vars['post_type'] ) && in_array( 'product', (array) $query->query_vars['post_type'] ) ) {
			// Query is for product post type
		} elseif ( $query->is_search() ) {
			// Search results page
		} elseif ( $query->is_tax( get_object_taxonomies( 'product' ) ) ) {
			// Product category, product tag or other archive (e.g. product attribute)
		} elseif ( $query->get_queried_object() && ( $query->is_post_type_archive( 'product' ) || $query->is_page( wc_get_page_id( 'shop' ) ) ) ) {
			// Main shop page
		} else {
			return;
		}

		// This is an action rather than a filter, so we don't need to return anything. $query is passed by reference.
		$query->query_vars = self::build_tax_query( $query->query_vars );
	}

	public static function protect_products_in_shortcode( $query_args, $atts, $loop_name = false ) {

		// Exclude protected categories from query used in shortcode
		$query_args = self::build_tax_query( $query_args );

		if ( 'product' === $loop_name && ! empty( $query_args['p'] ) ) {
			$query_args['post__in'] = array( $query_args['p'] );
			unset( $query_args['p'] );
		}
		return $query_args;
	}

	public static function protect_product_categories( $args, $taxonomies ) {

		// Check we have the right taxonomy
		if ( ! in_array( 'product_cat', (array) $taxonomies ) ) {
			return $args;
		}

		// Bail if our internal flag is set ('wc_ppc')
		if ( ! empty( $args['wc_ppc'] ) ) {
			return $args;
		}

		// Bail if we're getting the terms for one or more objects
		if ( ! empty( $args['object_ids'] ) ) {
			return $args;
		}

		// Bail if 'get' => 'all' set, e.g. when getting the term hierarchy or get_term_by() called
		if ( 'all' === $args['get'] || 1 === $args['number'] ) {
			return $args;
		}

		// Bail if we're building the exclusions list within get_terms() itself.
		// get_terms() is recursive when 'exclude_tree' is specified so we need to check here to avoid an infinite loop.
		if ( ! empty( $args['child_of'] ) && 'ids' === $args['fields'] && ! $args['hide_empty'] ) {
			return $args;
		}

		// Fix WP bug where 'include' is set to the non-empty array "Array ( [0] => null )"
		if ( is_array( $args['include'] ) ) {
			$args['include'] = array_filter( $args['include'] );
		}

		// 'exclude_tree' is ignored if include is set, so no point proceeding with our exclusions
		if ( ! empty( $args['include'] ) ) {
			return $args;
		}

		// Got this far, so we can exclude hidden categories from the terms query
		if ( $hidden = self::hidden_categories() ) {
			$args['exclude_tree'] = $hidden;
		}

		return $args;
	}

	public static function update_category_counts( $terms, $taxonomies, $args ) {
		// Bail if there are no terms, we have the wrong taxonomy, we're not padding term counts, or we're not returning term objects
		if ( ! $terms || ! in_array( 'product_cat', $taxonomies ) || empty( $args['pad_counts'] ) || ( 'all' !== $args['fields'] && 'all_with_object_id' !== $args['fields'] ) ) {
			return $terms;
		}

		// Adjust category counts - loop through each top-level category or sub-category
		foreach ( $terms as $term ) {
			if ( $hidden_descendants = array_intersect( self::hidden_categories(), get_term_children( $term->term_id, 'product_cat' ) ) ) {
				// Get number of (unique) products in the hidden child categories
				$count = count( array_unique( get_objects_in_term( $hidden_descendants, 'product_cat' ) ) );

				if ( $count && $count <= $term->count ) {
					$term->count = $term->count - $count;
				}
			}
		}

		return $terms;
	}

	public static function protect_nav_menus( $menu_items, $menu, $args ) {
		$removed_items = array();

		// Back-compat - default nav menu visibility to the same as main visibility setting
		$menu_vis_default	 = get_option( 'wc_ppc_show_protected', 'yes' );
		$show_protected		 = get_option( 'wc_ppc_show_protected_menu', $menu_vis_default ) === 'yes';

		foreach ( $menu_items as $key => $menu_item ) {

			if ( 'product' === $menu_item->object ) {
				// Menu item is a product
				if ( $product_cats = WC_PPC_Util::get_the_category_visibility( $menu_item->object_id ) ) {

					foreach ( $product_cats as $category ) {
						if ( $category->is_private( true ) || ( ! $show_protected && $category->is_protected( true ) ) ) {
							$removed_items[] = $menu_item->ID;
							unset( $menu_items[$key] );
							break;
						}
					}
				}
			} elseif ( 'product_cat' === $menu_item->object ) {
				// Menu item is a product category
				$category = WC_PPC_Util::get_category_visibility( $menu_item->object_id );

				if ( $category->is_private( true ) || ( ! $show_protected && $category->is_protected( true ) ) ) {
					$removed_items[] = $menu_item->ID;
					unset( $menu_items[$key] );
				}
			}
		} // foreach menu item
		// Now find and remove any children of any removed menu item - simples!
		while ( $removed_items ) {
			$child_items_removed = array();

			foreach ( $menu_items as $key => $menu_item ) {

				if ( in_array( $menu_item->menu_item_parent, $removed_items ) ) {
					$child_items_removed[] = $menu_item->ID;
					unset( $menu_items[$key] );
				}
			}

			// Update the removed list with the removed child items and start over
			$removed_items = $child_items_removed;
		}

		return array_values( $menu_items );
	}

	public static function product_is_purchasable( $purchasable, $product ) {

		$product_id		 = $product->is_type( 'variation' ) ? WC_PPC_Util::get_parent_id( $product ) : WC_PPC_Util::get_id( $product );
		$product_cats	 = WC_PPC_Util::get_the_category_visibility( $product_id );

		if ( $purchasable && $product_cats ) {
			foreach ( $product_cats as $category ) {
				if ( $category->is_protected( true ) || $category->is_private( true ) ) {
					$purchasable = false;
					break;
				}
			}
		}

		return $purchasable;
	}

	private static function build_tax_query( $query_args ) {

		if ( ! self::hidden_categories() ) {
			return $query_args;
		}

		$tax_query = array();

		// Is there already a tax query present?
		if ( ! empty( $query_args['tax_query'] ) ) {
			$tax_query = $query_args['tax_query'];

			// If current tax query is an 'OR' we need to nest it and wrap our query outside with an 'AND' relation
			if ( ! empty( $tax_query['relation'] ) && ( 'OR' === $tax_query['relation'] ) && count( $tax_query ) > 2 ) {
				$tax_query = array( $tax_query );
			}

			$tax_query = array( 'relation' => 'AND' ) + $tax_query;
		}

		$visibility_tax_query = array(
			'taxonomy'	 => 'product_cat',
			'field'		 => 'term_id',
			'terms'		 => self::hidden_categories(),
			'operator'	 => 'NOT IN'
		);

		if ( $unlocked = self::unlocked_categories() ) {
			$visibility_tax_query				 = array( $visibility_tax_query );
			$visibility_tax_query['relation']	 = 'OR';
			$visibility_tax_query[]				 = array(
				'taxonomy'	 => 'product_cat',
				'field'		 => 'term_id',
				'terms'		 => $unlocked,
				'operator'	 => 'IN'
			);
		}

		$tax_query[]			 = $visibility_tax_query;
		$query_args['tax_query'] = $tax_query;

		return $query_args;
	}

	private static function hidden_categories() {
		if ( false === self::$hidden_categories ) {
			self::$hidden_categories = array();
			$show_protected			 = WC_PPC_Util::showing_protected_categories();

			// Get all the product categories, and check which are hidden.
			foreach ( WC_PPC_Util::to_category_visibilities( WC_PPC_Util::get_product_categories() ) as $category ) {
				if ( $category->is_private() || ( ! $show_protected && $category->is_protected() ) ) {
					self::$hidden_categories[] = $category->term_id;
				}
			}
		}
		return self::$hidden_categories;
	}

	private static function unlocked_categories() {
		if ( false === self::$unlocked_categories ) {
			self::$unlocked_categories = array();

			// Get all the product categories, and check which are unlocked.
			foreach ( WC_PPC_Util::to_category_visibilities( WC_PPC_Util::get_product_categories() ) as $category ) {
				if ( $category->is_unlocked() ) {
					self::$unlocked_categories[] = $category->term_id;
				}
			}
		}
		return self::$unlocked_categories;
	}

}
// class WC_PPC_Category_Protector

