<?php
/**
 * The main plugin file for WooCommerce Protected Categories.
 *
 * This file is included during the WP bootstrap if the plugin is active.
 *
 * @wordpress-plugin
 * Plugin Name:       WooCommerce Protected Categories
 * Plugin URI:        https://barn2.co.uk/wordpress-plugins/woocommerce-password-protected-categories/
 * Description:       Protect your WooCommerce categories by password, user role or user account.
 * Version:           2.0
 * Author:            20script
 * Author URI:        https://www.20script.ir
 * Text Domain:       wc-cat-protect
 * Domain Path:       /languages
 *
 * WC requires at least: 2.4
 * WC tested up to: 3.4.4
 *
 * Copyright:		  Barn2 Media Ltd
 * License:           GNU General Public License v3.0
 * License URI:       http://www.gnu.org/licenses/gpl-3.0.html
 */
// Prevent direct file access
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * The main plugin class for WooCommerce Protected Categories.
 *
 * @package   WooCommerce_Protected_Categories
 * @author    Barn2 Media <info@barn2.co.uk>
 * @license   GPL-3.0
 * @copyright Barn2 Media Ltd
 */
final class WC_Protected_Categories_Plugin {

	//@todo: Update Download name in EDD
	const NAME	 = 'WooCommerce Protected Categories';
	const VERSION	 = '2.0';
	const FILE	 = __FILE__;

	// Our plugin license
	private $license;
	// The singleton instance
	private static $_instance = null;

	private function __construct() {
		$this->define_constants();
		$this->includes();
		add_action( 'plugins_loaded', array( $this, 'init_hooks' ) );

		$this->license = new Barn2_Plugin_License( self::FILE, self::NAME, self::VERSION, 'wc_ppc' );
	}

	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	private function define_constants() {
		if ( ! defined( 'WC_PPC_INCLUDES_DIR' ) ) {
			define( 'WC_PPC_INCLUDES_DIR', plugin_dir_path( self::FILE ) . 'includes/' );
		}
		if ( ! defined( 'WC_PPC_PLUGIN_BASENAME' ) ) {
			define( 'WC_PPC_PLUGIN_BASENAME', plugin_basename( self::FILE ) );
		}
		if ( ! defined( 'WC_PPC_COOKIE_PREFIX' ) ) {
			// Cookie has to be same one used by WP for password protected posts, as all caching plugins exclude it.
			define( 'WC_PPC_COOKIE_PREFIX', 'wp-postpass_' );
		}
	}

	private function includes() {
		// License
		require_once WC_PPC_INCLUDES_DIR . 'license/class-b2-plugin-license.php';

		// Core plugin
		require_once WC_PPC_INCLUDES_DIR . 'class-wc-ppc-util.php';
		require_once WC_PPC_INCLUDES_DIR . 'class-wc-ppc-category-visibility.php';
		require_once WC_PPC_INCLUDES_DIR . 'class-wc-ppc-login-shortcode.php';

		// Front end
		if ( $this->is_front_end() ) {
			require_once WC_PPC_INCLUDES_DIR . 'class-wc-ppc-password-form.php';
			require_once WC_PPC_INCLUDES_DIR . 'class-wc-ppc-loop-buffer.php';
			require_once WC_PPC_INCLUDES_DIR . 'class-wc-ppc-template-handler.php';
			require_once WC_PPC_INCLUDES_DIR . 'class-wc-ppc-category-protector.php';
			require_once WC_PPC_INCLUDES_DIR . 'class-wc-ppc-category-prefixer.php';
			require_once WC_PPC_INCLUDES_DIR . 'class-wc-ppc-template-functions.php';
			require_once WC_PPC_INCLUDES_DIR . 'compat/class-wc-ppc-theme-compat.php';
		}

		// Admin
		if ( $this->is_admin() ) {
			require_once WC_PPC_INCLUDES_DIR . 'admin/class-wc-ppc-admin-general.php';
			require_once WC_PPC_INCLUDES_DIR . 'admin/class-wc-ppc-admin-settings-page.php';
			require_once WC_PPC_INCLUDES_DIR . 'admin/class-wc-ppc-admin-category-visibility.php';
		}
	}

	public function init_hooks() {
		// Don't init plugin if WooCommerce isn't installed/active
		if ( ! class_exists( 'WooCommerce' ) ) {
			return;
		}

		add_action( 'after_setup_theme', array( $this, 'after_setup_theme' ) );
		add_action( 'init', array( $this, 'init' ) );
	}

	public function after_setup_theme() {
		if ( $this->license->is_valid() && $this->is_front_end() ) {
			WC_PPC_Template_Functions::load_template_overrides();
			WC_PPC_Theme_Compat::register_theme_hooks();
		}
	}

	public function init() {
		$this->load_textdomain();

		if ( $this->is_admin() ) {
			new WC_PPC_Admin_General();
			new WC_PPC_Admin_Settings_Page( $this->license );
		}

		if ( $this->license->is_valid() ) {
			// License is valid so we can register shortcode, setup visibility field, etc
			if ( $this->is_admin() ) {
				new WC_PPC_Admin_Category_Visibility();
			}

			// Front-end
			if ( $this->is_front_end() ) {
				WC_PPC_Login_Shortcode::register_shortcode();
				WC_PPC_Password_Form::register_login_handler();
				WC_PPC_Template_Handler::register_template_hooks();
				WC_PPC_Category_Protector::register_protection_hooks();
				WC_PPC_Category_Prefixer::register_prefix_hooks();
			}
		}
	}

	private function load_textdomain() {
		load_plugin_textdomain( 'wc-cat-protect', false, dirname( self::FILE ) . '/languages' );
	}

	private function is_admin() {
		return is_admin() || ( defined( 'WP_CLI' ) && WP_CLI );
	}

	private function is_front_end() {
		return ! is_admin() || ( defined( 'DOING_AJAX' ) && DOING_AJAX );
	}

}
// end class WC_Protected_Categories_Plugin

/**
 * Helper function to return the plugin instance.
 *
 * @return WC_Protected_Categories_Plugin
 */
function WC_PPC() {
	return WC_Protected_Categories_Plugin::instance();
}
// Load the plugin
WC_Protected_Categories_Plugin::instance();
